/* %Z%%M% %I% %W% %G% %U%
 * ===========================================================================
 * Licensed Materials - Property of IBM
 * All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 * 5724-S24
 * (C) Copyright IBM Corp. 2010
 * ===========================================================================
 */
package com.ibm.tscc.rtss.authz.sample.finder.xpath;

import java.io.StringWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import java.util.logging.Logger;

import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.apache.xml.serialize.DOMSerializer;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import com.ibm.tscc.rtss.authz.api.Attribute;
import com.ibm.tscc.rtss.authz.api.AttributeFactory;
import com.ibm.tscc.rtss.authz.api.AttributeInstantiationException;
import com.ibm.tscc.rtss.authz.api.IExternalFinder;
import com.ibm.tscc.rtss.authz.api.RequestContext;

/**
 * <p>
 * The XPath Attribute Finder extracts data from the Request's ResourceContent
 * and returns it as an attribute.
 * 
 * <p>
 * This is needed to enable PIP chaining when the original source is in the SOAP Body
 * (or another piece of provided XML).  We can't do this using the "In the request,
 * in a non-standard location" of the Tivoli Security Policy Manager policy configuration as that modifies the
 * policy itself (uses XACML's AttributeSelector).
 * 
 * <p>
 *  The finder requires three configuration parameters in the security-services.xmi:
 *   <ul>
 *   <li>xpath -- the XPath to use on the ResourceContent.</li>
 *   <li>return.attributeId -- which XACML AttributeId to return the value as.</li>
 *   <li>return.section -- which XACML request section to return the found data in.</li>
 *   </ul>
 * 
 * <p>
 * The following configuration is required for the security-services.xmi file in the AttributeRetrievalServices
 * section:
 * 
 * <pre>
 * {@code
 * 	<items name="XPath PIP">
 *          <properties>
 *           <values name="type" value="custom" type="java.lang.String"/>
 *           <values name="id" value="com.ibm.tscc.rtss.authz.sample.finder.xpath.XPathAttributeFinder" type="java.lang.String"/>
 *           <values name="enabled" value="true" type="java.lang.String"/>
 *           <values name="xpath" value="//*[local-name()='Foo']/text()" type="java.lang.String"/>
 *           <values name="return.attributeId" value="urn:xpath:attr" type="java.lang.String"/>
 *           <values name="return.section" value="Resource" type="java.lang.String"/>
 *         </properties>
 *  </items>
 *  }
 *  </pre>
 *
 */
@SuppressWarnings("deprecation")
public class XPathAttributeFinder implements IExternalFinder {

    public static final String XPathAttributeFinder_java_sourceCodeID = "$Id: %Z%%M%    %I%  %W%  %G%  %U% $";
    public static final String XPathAttributeFinder_java_copyright = "Licensed Materials - Property of IBM - All Rights Reserved. 5724-S24  (C) Copyright IBM Corp. 2010";

	private static final Logger _log = Logger.getLogger( XPathAttributeFinder.class.getName() );
	
	private boolean _enabled = false;
	private String _attributeId = null;
	private IExternalFinder.RequestSection _section = null;
	
	private XPathExpression _xpath = null;

	public void init(Properties props) {
		
		_enabled = Boolean.valueOf( props.getProperty(PROP_ENABLED));
		_attributeId = props.getProperty( PROP_RETURN_ATTRIBUTE_ID );
		String returnSection = props.getProperty( PROP_RETURN_SECTION );
		
		if ( _attributeId == null || returnSection == null ) {
			throw new IllegalArgumentException( "Missing required configuration parameters." );
		}
		
		_section = IExternalFinder.RequestSection.valueOf( returnSection );
		
		String xPath = props.getProperty( "xpath" );
		
		try {
			XPathFactory xpf = XPathFactory.newInstance();
			XPath xpath = xpf.newXPath();
			_xpath = xpath.compile( xPath );
		} catch ( XPathExpressionException e ) {
			throw new IllegalArgumentException( "Invalid XPath", e );
		}
	}

	
	public List<Attribute> getActionAttributes(RequestContext context, String attributeId, String dataType, String issuer) {
		List<Attribute> retVal = new ArrayList<Attribute>();
		
		if ( hasActionAttributes(attributeId, dataType, issuer)) {
			retVal.addAll( getAttributes( context, attributeId, dataType, issuer ) );
		}
		
		return retVal;
	}

	public List<Attribute> getEnvironmentAttributes(RequestContext context, String attributeId, String dataType, String issuer) {
		List<Attribute> retVal = new ArrayList<Attribute>();
		
		if ( hasEnvironmentAttributes(attributeId, dataType, issuer)) {
			retVal.addAll( getAttributes( context, attributeId, dataType, issuer ) );
		}
		
		return retVal;
	}

	public List<Attribute> getResourceAttributes(RequestContext context, String attributeId, String dataType, String issuer) {
		List<Attribute> retVal = new ArrayList<Attribute>();
		
		if ( hasResourceAttributes(attributeId, dataType, issuer)) {
			retVal.addAll( getAttributes( context, attributeId, dataType, issuer ) );
		}
		
		return retVal;
	}

	public List<Attribute> getSubjectAttributes(RequestContext context, String attributeId, String dataType, String issuer, String subjectCategory) {
		List<Attribute> retVal = new ArrayList<Attribute>();
		
		if ( hasSubjectAttributes(attributeId, dataType, issuer, subjectCategory)) {
			retVal.addAll( getAttributes( context, attributeId, dataType, issuer ) );
		}
		
		return retVal;	
	}

	public boolean hasActionAttributes(String attributeId, String dataType, String issuer) {
		return hasAttribute( RequestSection.Action, attributeId, dataType, issuer );
	}

	public boolean hasEnvironmentAttributes(String attributeId,
			String dataType, String issuer) {
		return hasAttribute( RequestSection.Environment, attributeId, dataType, issuer );
	}

	public boolean hasResourceAttributes(String attributeId, String dataType,
			String issuer) {
		return hasAttribute( RequestSection.Resource, attributeId, dataType, issuer );
	}

	public boolean hasSubjectAttributes(String attributeId, String dataType,
			String issuer, String subjectCategory) {
		return hasAttribute( RequestSection.Subject, attributeId, dataType, issuer );
	}
	
	private boolean hasAttribute( IExternalFinder.RequestSection section, String attributeId, String dataType, String issuer )  {
		return _enabled && _section.equals( section ) && _attributeId.equals( attributeId );
	}
	
	/*
	 * Do the XPath on the request context, and return created attributes.
	 */
	private List<Attribute> getAttributes( RequestContext context, String attributeId, String dataType, String issuer ) {
		
		List<Attribute> retVal = new ArrayList<Attribute>();
		
		Node[] resourceContents = context.getResourceContents();
		
		for ( Node thisNode : resourceContents ) {
			
			if ( thisNode.getNodeType() == Node.ELEMENT_NODE ) {
				try {
					_log.finest( "Executing XPath '" + _xpath.toString() + "' on element: " ) ;
					_log.finest( nodeToString( (Element) thisNode ));
					
					String foundData = _xpath.evaluate(thisNode);
					Attribute newAttr = AttributeFactory.createAttribute(attributeId, dataType, foundData);
					retVal.add( newAttr );
				} catch ( XPathExpressionException e ) {
					e.printStackTrace();
				} catch ( AttributeInstantiationException e ) {
					e.printStackTrace();
				}
			}
		}
		
		return retVal;
	}
	
	 /*
	 * Create a nicely formatted string representation of an Element. Used
	 * mostly for logging and testing purposes.
	 * 
	 */
	private static String nodeToString(Element input) {
		if (input == null)
			return "";

		String processedXML = null;
		try {
			OutputFormat format = new OutputFormat();
			format.setIndenting(true);

			StringWriter sw = new StringWriter();

			XMLSerializer sz = new XMLSerializer(sw, format);
			sz.setNamespaces(true);
			DOMSerializer dw = sz.asDOMSerializer();
			dw.serialize((Element) input);

			processedXML = sw.getBuffer().toString();
		} catch (Exception ex) {
			// Ignore, empty string will be returned
		}
		return processedXML;
	}
}
