/* %Z%%M% %I% %W% %G% %U%
 * ===========================================================================
 * Licensed Materials - Property of IBM
 * All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 * 5724-S24
 * (C) Copyright IBM Corp. 2010
 * ===========================================================================
 */
package com.ibm.tscc.rtss.authz.sample.finder;

import java.util.List;
import java.util.Properties;

import java.util.LinkedList;

import com.ibm.tscc.rtss.authz.api.Attribute;
import com.ibm.tscc.rtss.authz.api.AttributeFactory;
import com.ibm.tscc.rtss.authz.api.IExternalFinder;
import com.ibm.tscc.rtss.authz.api.RequestContext;

/**
 * <p> 
 * This an example of how to write an attribute finder for looking up external 
 * attribute information for the request context.
 *
 * <p>
 * The finder requires the following configuration parameter in the security-services.xmi:
 * <ul>
 * <li>VALUE -- the value of the attribute returned</li>
 * </ul>
 * 
 * <p>
 * The following configuration is required in the security-services.xmi file, in the AttributeRetrievalServices
 * section:
 *  <pre>
 * {@code
 *	<items name="Java PIP">
 *     <properties>
 *        <values name="type" value="custom" type="java.lang.String"/>
 *        <values name="id" value="com.ibm.tscc.rtss.authz.sample.finder.SampleCustomAttributeFinder" type="java.lang.String"/>
 *        <values name="enabled" value="true" type="java.lang.String"/>
 *        <values name="VALUE" value="theAttributeValue" type="java.lang.String"/>
 *        <values name="return.attributeId" value="theAttributeToReturn" type="java.lang.String"/>
 *        <values name="return.section" value="Resource" type="java.lang.String"/>
 *     </properties>
 *  </items>
 * }
 * </pre>
 * 
 * Change the values above appropriate to your configuration.
 *
 */
public class SampleCustomAttributeFinder implements IExternalFinder
{
    public final static String SampleCustomAttributeFinder_java_sourceCodeID = "$Id: %Z%%M%    %I%  %W%  %G%  %U% $";
    public static final String SampleCustomAttributeFinder_java_copyright = "Licensed Materials - Property of IBM - All Rights Reserved. 5724-S24  (C) Copyright IBM Corp. 2010";
      
	private boolean enabled;
	private String attributeId;
	private IExternalFinder.RequestSection section;
	private static final String PROP_VALUE = "VALUE";
	
	private String value;
	
	public void init(Properties props) 
	{
		String enabledString = props.getProperty(IExternalFinder.PROP_ENABLED);
		
		try
		{
			enabled = Boolean.valueOf(enabledString);
		}
		catch (Exception e)
		{
			// This will happen if enabledString is null or contains gibberish
			enabled = false;
		}
		
		attributeId = props.getProperty(IExternalFinder.PROP_RETURN_ATTRIBUTE_ID);
		
		if (attributeId == null)
			attributeId = "";
		
		String sectionString = props.getProperty(IExternalFinder.PROP_RETURN_SECTION);
		
		try
		{
			section = IExternalFinder.RequestSection.valueOf(sectionString);
		}
		catch (Exception e)
		{
			
			section = IExternalFinder.RequestSection.Subject;
		}
		
		value = props.getProperty(PROP_VALUE);
		
		System.out.println("SampleCustomAttributeFinder initializing...");
		System.out.println("    enabled: " + enabled);
		System.out.println("    attribute id: " + attributeId);
		System.out.println("    section: " + section.toString());
		System.out.println("    value: " + value);
		
		generateExampleAttributes();
	}
	
	/**
	 * This operation will demonstrate how to instantiate an attribute of each type
	 * and give bounds to the acceptable values for each 
	 */
	private void generateExampleAttributes()
	{
		String representation = null;
		
		/**
		 * Type:   STRING
		 * Values: Any non-null String
		 */
		try
		{
			representation = "The quick brown fox jumps over the lazy dog.";
			AttributeFactory.createAttribute(attributeId, 
					AttributeFactory.DataType.STRING, 
					representation);
		}
		catch (Exception e)
		{
			System.out.println("Creation of a STRING attribute failed!");
			System.out.println("The value '" + representation + "' was not valid!");
			e.printStackTrace();
		}
		
		/**
		 * Type:   BOOLEAN
		 * Values: Any non-null String
		 * Result: True  --> any String equivalent to 'true', ignoring case
		 *         False --> anything else
		 */
		try
		{
			representation = "true";
			AttributeFactory.createAttribute(attributeId, 
					AttributeFactory.DataType.BOOLEAN, 
					representation);
		}
		catch (Exception e)
		{
			System.out.println("Creation of a BOOLEAN attribute failed!");
			System.out.println("The value '" + representation + "' was not valid!");
			e.printStackTrace();
		}
		
		/**
		 * Type:   INTEGER
		 * Values: A non-null String consisting of numeric characters, [0,9]
		 *         May optionally begin with a minus sign(-) to indicate negative values
		 *         The number must be within the bounds of the primitive type int
		 *         no letters
		 *         no symbols, except as noted above
		 */
		try
		{
			representation = "34";
			AttributeFactory.createAttribute(attributeId, 
					AttributeFactory.DataType.INTEGER, 
					representation);
		}
		catch (Exception e)
		{
			System.out.println("Creation of a INTEGER attribute failed!");
			System.out.println("The value '" + representation + "' was not valid!");
			e.printStackTrace();
		}
		
		/**
		 * Type:   DOUBLE
		 * Values: A non-null String consisting of numeric characters, [0,9]
		 *         May optionally begin with a minus sign(-) to indicate negative values
		 *         May be expressed with e notation (e.g. 10e1 = 10 and 10e10 = 100)
		 *         May contain a single deci 
		 */
		try
		{
			representation = "34.43";
			AttributeFactory.createAttribute(attributeId, 
					AttributeFactory.DataType.DOUBLE, 
					representation);
		}
		catch (Exception e)
		{
			System.out.println("Creation of a DOUBLE attribute failed!");
			System.out.println("The value '" + representation + "' was not valid!");
			e.printStackTrace();
		}
		
		/**
		 * Type:   DATE_TIME
		 * Values: A non-null String that conforms to the regular expression:
		 *         '-'? yyyy '-' mm '-' dd 'T' hh ':' mm ':' ss ('.' s+)? ((('+' | '-') hh ':' mm) | 'Z')?
		 *          year|        |      |      |      |      |      |                   |
		 *                  month|      |      |      |      |      |                   |
		 *                           day|      |      |      |      |                   |
		 *                                 hour|      |      |      |                   |
		 *                                      minute|      |      |                   |
		 *                                             second|      |                   |
		 *                                     fractions of a second|                   |
		 *                                                                      timezone|
		 */
		try
		{
			representation = "2008-10-19T00:00:00-06:00";
			AttributeFactory.createAttribute(attributeId, 
					AttributeFactory.DataType.DATE_TIME, 
					representation);
		}
		catch (Exception e)
		{
			System.out.println("Creation of a DATE_TIME attribute failed!");
			System.out.println("The value '" + representation + "' was not valid!");
			e.printStackTrace();
		}
		
		/**
		 * Type:   DATE
		 * Values: A non-null String that conforms to the regular expression:
		 *         '-'? yyyy '-' mm '-' dd ((('+' | '-') hh ':' mm) | 'Z')?
		 *          year|        |      |                |
		 *                  month|      |                |
		 *                           day|                |
		 *                                       timezone|
		 */
		try
		{
			representation = "2000-07-23";
			AttributeFactory.createAttribute(attributeId, 
					AttributeFactory.DataType.DATE, 
					representation);
		}
		catch (Exception e)
		{
			System.out.println("Creation of a DATE attribute failed!");
			System.out.println("The value '" + representation + "' was not valid!");
			e.printStackTrace();
		}
		
		/**
		 * Type:   TIME
		 * Values: A non-null String that conforms to the regular expression:
		 *         hh ':' mm ':' ss ('.' s+)? ((('+' | '-') hh ':' mm) | 'Z')?
		 *     hour|      |      |      |                   |
		 *          minute|      |      |                   |
		 *                 second|      |                   |
		 *         fractions of a second|                   |
		 *                                          timezone|
		 */
		try
		{
			representation = "14:34:22.766-01:30";
			AttributeFactory.createAttribute(attributeId, 
					AttributeFactory.DataType.TIME, 
					representation);
		}
		catch (Exception e)
		{
			System.out.println("Creation of a TIME attribute failed!");
			System.out.println("The value '" + representation + "' was not valid!");
			e.printStackTrace();
		}
		
		/**
		 * Type:   ANY_URI
		 * Values: A non-null String that conforms to the standard URI syntax,
		 *         defined by [RFC 2396] and amended by [RFC 2732].
		 */
		try
		{
			representation = "file:///C:/MyDir/MyFile.txt";
			AttributeFactory.createAttribute(attributeId, 
					AttributeFactory.DataType.ANY_URI, 
					representation);
		}
		catch (Exception e)
		{
			System.out.println("Creation of a ANY_URI attribute failed!");
			System.out.println("The value '" + representation + "' was not valid!");
			e.printStackTrace();
		}
	}
	
	/**
	 * For the purposes of this sample, just return the value from the RTSS configuration
	 */
	private List<Attribute> createReturnAttributes(RequestContext context,
			String attributeId, String dataType, String issuer)
	{
		List<Attribute> attrList = new LinkedList<Attribute>();
		try
		{
			attrList.add(AttributeFactory.createAttribute(attributeId, dataType, value));
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
		
		System.out.println("SAMPLE CUSTOM ATTRIBUTE FINDER -- returning: " + attrList);
		
		return attrList;
	}
	
	public List<Attribute> getActionAttributes(RequestContext context,
			String attributeId, String dataType, String issuer) 
	{
		return createReturnAttributes(context, attributeId, dataType, issuer);
	}

	public List<Attribute> getEnvironmentAttributes(RequestContext context,
			String attributeId, String dataType, String issuer) 
	{
		return createReturnAttributes(context, attributeId, dataType, issuer);
	}

	public List<Attribute> getResourceAttributes(RequestContext context,
			String attributeId, String dataType, String issuer) 
	{
		return createReturnAttributes(context, attributeId, dataType, issuer);
	}

	public List<Attribute> getSubjectAttributes(RequestContext context,
			String attributeId, String dataType, String issuer,
			String subjectCategory) 
	{
		return createReturnAttributes(context, attributeId, dataType, issuer);
	}

	public boolean hasActionAttributes(String attributeId, String dataType,
			String issuer) 
	{
		// Don't worry about issuer right now
		return enabled && 
				section == IExternalFinder.RequestSection.Action && 
				attributeId.equals(this.attributeId);
	}

	public boolean hasEnvironmentAttributes(String attributeId,
			String dataType, String issuer) 
	{
		// Don't worry about issuer right now
		return enabled && 
				section == IExternalFinder.RequestSection.Environment && 
				attributeId.equals(this.attributeId);
	}

	public boolean hasResourceAttributes(String attributeId, String dataType,
			String issuer) 
	{
		// Don't worry about issuer right now
		return enabled && 
				section == IExternalFinder.RequestSection.Resource && 
				attributeId.equals(this.attributeId);
	}

	public boolean hasSubjectAttributes(String attributeId, String dataType,
			String issuer, String subjectCategory) 
	{
		// Don't worry about issuer right now
		return enabled && 
				section == IExternalFinder.RequestSection.Subject && 
				attributeId.equals(this.attributeId);
	}
}
