/* %Z%%M% %I% %W% %G% %U%
 * ===========================================================================
 * Licensed Materials - Property of IBM
 * All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 * 5724-S24
 * (C) Copyright IBM Corp. 2010
 * ===========================================================================
 */
package com.ibm.tscc.rtss.jaccplus.sample.authz;

import java.io.IOException;
import java.io.PrintWriter;
import java.security.Permission;
import java.security.Principal;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.ibm.sec.authz.jaccplus.ApplicationEvaluationContext;
import com.ibm.sec.authz.jaccplus.ApplicationPermission;
import com.ibm.sec.authz.jaccplus.ApplicationPolicy;
import com.ibm.sec.authz.jaccplus.ApplicationSubject;
import com.ibm.sec.authz.jaccplus.ApplicationSubjectContext;
import com.ibm.sec.authz.jaccplus.ApplicationUserPrincipal;


/**
 * <p>
 * Servlet implementation class AuthorizeServlet.  This sample shows how to use 
 * the JACCPLUS API for authorization decisions.
 *   
 * <p> For this sample, create a custom application service in Tivoli Security Policy Manager (TSPM):
 * <pre>
 *    MyAppService
 *       Element1
 *          Element11
 *       Element2 
 *          Element22
 * </pre>
 * 
 * <p>
 * Default values used for authorization:
 *    <ul>
 *       <li>Context ID: MyAppServiceID</li>
 *       <li>Resource ID: MyAppServiceResourceID</li>
 *       <li>Action: invoke</li>
 *    </ul>
 *    
 * <p>
 * Create TSPM policy for the service with various resources and actions mapped to groups and distribute to Tivoli Runtime Security Services (RTSS).  
 * The AuthorizeServlet sample can be used to drive the IExternalRule and IExternalFinder samples included 
 * in the SDK by creating TSPM policy with appropriate rules that use them. 
 * This application should be installed where the RTSS client is installed.
 * 
 * <p>
 * To execute sample, bring up browser with url:<br>
 * http://localhost:9080/rtss/sample/jaccplus/index.jsp
 * 
 */
 public class AuthorizeServlet extends javax.servlet.http.HttpServlet implements javax.servlet.Servlet 
 {
    public static final String AuthorizeServlet_java_sourceCodeID = "$Id: %Z%%M%    %I%  %W%  %G%  %U% $";
    public static final String AuthorizeServlet_java_copyright = "Licensed Materials - Property of IBM - All Rights Reserved. 5724-S24  (C) Copyright IBM Corp. 2010";

    public static final String APP_EVAL_CTX = "com.ibm.sec.authz.jaccplus.ApplicationEvaluationContext";
    
    private static final long serialVersionUID = -173170212646560987L;

    /* (non-Java-doc)
     * @see javax.servlet.http.HttpServlet#HttpServlet()
     */
    public AuthorizeServlet() {
        super();
    }       
    
    public void init() throws ServletException {
      	ApplicationEvaluationContext _evalCtx = new ApplicationEvaluationContext();
    	this.getServletContext().setAttribute(APP_EVAL_CTX, _evalCtx);
    }
    
    
    /* (non-Java-doc)
     * @see javax.servlet.http.HttpServlet#doGet(HttpServletRequest request, HttpServletResponse response)
     */
    protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException 
    {
        doIt(request, response);
    }   
    
    /* (non-Java-doc)
     * @see javax.servlet.http.HttpServlet#doPost(HttpServletRequest request, HttpServletResponse response)
     */
    protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException 
    {
        doIt(request, response);
    } 
    
    private void doIt(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
    {
        String contextID = request.getParameter("contextID");
        String groups = request.getParameter("groups");   // group names separated by space
        String resourceID = request.getParameter("resourceID");
        String action = request.getParameter("action");
        String[] realGroups;
        if (groups != null)
        {
            realGroups = groups.split(" ");
        }
        else
        {
            realGroups = new String[] {};
        }
        
        if (contextID == null || contextID.length() < 1)
        {
            contextID = "MyAppServiceID";
        }
        
        if (resourceID == null || resourceID.length() < 1)
        {
        	resourceID = "MyAppServiceResourceID";
        }
        
        if (action == null || action.length() < 1) 
        {
        	action = "invoke";
        }
        
            
        PrintWriter     out;
        String      title = "Authorize Servlet Output";

        // set content type and other response header fields first
        response.setStatus(HttpServletResponse.SC_OK);
        response.setContentType("text/html");

        // then write the data of the response
        out = response.getWriter();

        out.println("<HTML><HEAD><TITLE>");
        out.println(title);
        out.println("</TITLE></HEAD><BODY>");
        out.println("<H1>" + title + "</H1>");
        out.println("<P>This is the result from authorization : ");
        
        
        try 
        {
        	// When using ApplicationEvaluationContext, the subject information is provided
        	// by the application
            ApplicationEvaluationContext _evalCtx = (ApplicationEvaluationContext)this.getServletContext().getAttribute(APP_EVAL_CTX);
                  
            // This sample does no rely on WAS setting up the subject for us, so we over-ride the WAS stuff 
            // with manually provided Subject.
            ApplicationSubjectContext subjectContext = new ApplicationSubjectContext();
            _evalCtx.registerHandler( ApplicationSubjectContext.SUBJECT_KEY, subjectContext, true );
            _evalCtx.registerHandler( ApplicationSubjectContext.SUBJECT_ATTRIBUTES_KEY, subjectContext, true);
    
            ApplicationSubject subject = new ApplicationSubject();
                  
            Principal[] prins = new Principal[realGroups.length]; 
            for (int index = 0; index < realGroups.length; index++)
            {
                prins[index] = new ApplicationUserPrincipal(realGroups[index]);
            }
            
            subject.setGroupPrincipals(prins);
            
            Map<String, Object> handlerData = _evalCtx.getHandlerData();
            handlerData.put( "java.security.auth.Subject.container", subject );
            
            ApplicationPolicy policy = ApplicationPolicy.getPolicy();
            
            Permission perm = new ApplicationPermission(resourceID, action);
            
            out.println("<P>RESULT: " + policy.implies(contextID, _evalCtx, perm));
        } 
        catch (Exception e) 
        {
            out.println("<P>RESULT: " + e.toString());
        }
        
        out.println("</BODY></HTML>");
        out.close();
    }
    
}
